"use strict";
var fs = require('fs');
var path = require('path');
var async = require('async');
var child_process = require('child_process');

var fsPath = {
  _win32: process.platform === 'win32',
  _supportExecSync: function () {
    if (!child_process.execSync) {
      throw new Error('your node.js version is to low(<0.11.12).')
    }
    return this;
  },
  mkdir: function (dist, callback) {
    var that = this;
    dist = path.resolve(dist);
    fs.exists(dist, function (exists) {
      if (!exists) {
        if (that._win32) {
          fsPath.mkdir(path.dirname(dist), function (err) {
            if (err) {
              callback && callback(err);
            } else {
              fs.mkdir(dist, function (err) {
                if (err.code === 'EEXIST') {
                  err = null;
                }
                callback && callback(err);
              });
            }
          });
        } else {
          child_process.exec('mkdir -p ' + dist.replace(/ /g, '\\ '), function (error, stdout, stderr) {
            callback && callback(error);
          });
        }
      } else {
        callback && callback(null);
      }
    });
  },
  mkdirSync: function (dist) {
    dist = path.resolve(dist);
    if (!fs.existsSync(dist)) {
      fsPath.mkdirSync(path.dirname(dist));
      fs.mkdirSync(dist);
    }
  },
  copy: function (from, dist, callback) {
    var that = this,
      cmd = '';
    dist = path.resolve(dist);
    fs.lstat(from, function (err, stats) {
      if (err) {
        callback(err);
      } else {
        if (that._win32) {
          // windows
          if (stats.isDirectory()) {
            cmd = 'echo da|xcopy /s /e "' + path.join(from, '*') + '" "' + dist + '"';
          } else {
            cmd = 'echo fa|xcopy "' + from + '" "' + dist + '"';
          }
        } else {
          // linux or mac
          cmd = 'cp -r ' + from.replace(/ /g, '\\ ') + ' ' + dist.replace(/ /g, '\\ ');
        }
        child_process.exec(cmd, function (error, stdout, stderr) {
          callback && callback(error);
        });
      }
    });
  },
  copySync: function (from, dist) {
    var cmd = '';
    var stats = fs.lstatSync(from);
    dist = path.resolve(dist);
    if (this._win32) {
      // windows
      if (stats.isDirectory()) {
        cmd = 'echo da|xcopy /s /e "' + path.join(from, '*') + '" "' + dist + '"';
      } else {
        cmd = 'echo fa|xcopy "' + from + '" "' + dist + '"';
      }
    } else {
      // linux or mac
      cmd = 'cp -r ' + from.replace(/ /g, '\\ ') + ' ' + dist.replace(/ /g, '\\ ');
    }
    this._supportExecSync();
    child_process.execSync(cmd);
  },
  remove: function (from, callback) {
    var cmd = '';
    if (this._win32) {
      // windows
      cmd = 'rd /s /q "' + from + '"';
    } else {
      // linux or mac
      cmd = 'rm -rf "' + from + '"';
    }
    child_process.exec(cmd, function (error, stdout, stderr) {
      callback && callback(error);
    });
  },
  removeSync: function (from) {
    var cmd = '';
    if (this._win32) {
      // windows
      cmd = 'rd /s /q "' + from + '"';
    } else {
      // linux or mac
      cmd = 'rm -rf "' + from + '"';
    }
    this._supportExecSync();
    child_process.execSync(cmd);
  },
  find: function (from, filter, callback) {
    var filelist = {
      dirs: [],
      files: []
    };
    if (arguments.length < 3) {
      callback = filter;
      filter = null;
    }
    fs.readdir(from, function (err, files) {
      if (err) {
        callback && callback(err);
      } else {
        async.each(files, function (file, callback) {
          var filepath = path.join(from, file);
          fs.lstat(filepath, function (err, stats) {
            if (err) {
              callback(err);
            } else {
              if (stats.isDirectory()) {
                if (!filter || filter(filepath, 'directory', file)) {
                  filelist.dirs.indexOf(filepath) === -1 && filelist.dirs.push(filepath);
                  fsPath.find(filepath, filter, function (err, files) {
                    if (err) {
                      callback && callback(err);
                    } else {
                      files.dirs.forEach(function (_dir) {
                        filelist.dirs.indexOf(_dir) === -1 && filelist.dirs.push(_dir);
                      });
                      filelist.files = filelist.files.concat(files.files);
                      callback && callback(null);
                    }
                  });
                } else {
                  callback && callback(null);
                }
              } else {
                if (!filter || filter(filepath, 'file', file)) {
                  filelist.files.push(filepath);
                }
                callback && callback(null);
              }
            }
          });
        }, function (err) {
          if (err) {
            callback && callback(err);
          } else {
            callback && callback(null, filelist);
          }
        });
      }
    });
  },
  findSync: function (from, filter) {
    var filelist = {
      dirs: [],
      files: []
    };
    fs.readdirSync(from).forEach(function (file) {
      var filepath = path.join(from, file);
      var stats = fs.lstatSync(filepath);
      if (stats.isDirectory()) {
        if (!filter || filter(filepath, 'directory', file)) {
          filelist.dirs.indexOf(filepath) === -1 && filelist.dirs.push(filepath);
          var files = fsPath.findSync(filepath);
          files.dirs.forEach(function (_dir) {
            filelist.dirs.indexOf(_dir) === -1 && filelist.dirs.push(_dir);
          });
          filelist.files = filelist.files.concat(files.files);
        }
      } else {
        if (!filter || filter(filepath, 'file', file)) {
          filelist.files.push(filepath);
        }
      }
    });
    return filelist;
  },
  writeFile: function (dist, content, encoding, callback) {
    dist = path.resolve(dist);
    if (typeof encoding === 'function') {
      callback = encoding;
      encoding = 'utf-8';
    }
    fsPath.mkdir(path.dirname(dist), function (err) {
      if (err) {
        callback(err);
      } else {
        fs.writeFile(dist, content, {
          encoding: encoding
        }, callback);
      }
    });
  },
  writeFileSync: function (dist, content, encoding) {
    dist = path.resolve(dist);
    if (typeof encoding === 'function') {
      callback = encoding;
      encoding = 'utf-8';
    }
    fsPath.mkdirSync(path.dirname(dist));
    fs.writeFileSync(dist, content, {
      encoding: encoding
    });
  }
};
module.exports = fsPath;